import { gossipsub } from "@chainsafe/libp2p-gossipsub";
import { noise } from "@chainsafe/libp2p-noise";
import { yamux } from "@chainsafe/libp2p-yamux";
import { circuitRelayTransport } from "@libp2p/circuit-relay-v2";
import { identify } from "@libp2p/identify";
import { pubsubPeerDiscovery } from "@libp2p/pubsub-peer-discovery";
import { webRTC } from "@libp2p/webrtc";
import { webSockets } from "@libp2p/websockets";
import * as filters from "@libp2p/websockets/filters";
import { createLibp2p } from "libp2p";
import { multiaddr } from "multiaddr";
import { stringToStream } from "./stream";
export class TopologyNetworkNode {
    _config;
    _node;
    _pubsub;
    peerId = "";
    constructor(config) {
        this._config = config;
    }
    async start() {
        this._node = await createLibp2p({
            addresses: {
                listen: ["/webrtc"],
            },
            connectionEncryption: [noise()],
            connectionGater: {
                denyDialMultiaddr: () => {
                    return false;
                },
            },
            peerDiscovery: [pubsubPeerDiscovery()],
            services: {
                identify: identify(),
                pubsub: gossipsub(),
            },
            streamMuxers: [yamux()],
            transports: [
                webSockets({ filter: filters.all }),
                webRTC(),
                circuitRelayTransport({
                    discoverRelays: 1,
                }),
            ],
        });
        // bootstrap
        // TODO: use another technique instead of dial
        await this._node.dial([
            multiaddr("/ip4/127.0.0.1/tcp/50000/ws/p2p/Qma3GsJmB47xYuyahPZPSadh1avvxfyYQwk8R3UnFrQ6aP"),
        ]);
        this._pubsub = this._node.services.pubsub;
        this.peerId = this._node.peerId.toString();
        console.log("topology::network::start: Successfuly started topology network w/ peer_id", this.peerId);
    }
    subscribe(topic) {
        if (!this._node) {
            console.error("topology::network::subscribe: Node not initialized, please run .start()");
            return;
        }
        try {
            this._pubsub?.subscribe(topic);
            console.log("topology::network::subscribe: Successfuly subscribed the topic", topic);
        }
        catch (e) {
            console.error("topology::network::subscribe:", e);
        }
    }
    unsubscribe(topic) {
        if (!this._node) {
            console.error("topology::network::unsubscribe: Node not initialized, please run .start()");
            return;
        }
        try {
            this._pubsub?.unsubscribe(topic);
            console.log("topology::network::unsubscribe: Successfuly unsubscribed the topic", topic);
        }
        catch (e) {
            console.error("topology::network::unsubscribe:", e);
        }
    }
    async broadcastMessage(topic, message) {
        try {
            if (this._pubsub?.getSubscribers(topic)?.length === 0)
                return;
            await this._pubsub?.publish(topic, message);
            console.log("topology::network::broadcastMessage: Successfuly broadcasted message to topic", topic);
        }
        catch (e) {
            console.error("topology::network::broadcastMessage:", e);
        }
    }
    async sendMessage(peerId, protocols, message) {
        const connection = await this._node?.dial([multiaddr(peerId)]);
        const stream = (await connection?.newStream(protocols));
        stringToStream(stream, message);
    }
    async sendMessageRandomTopicPeer(topic, protocols, message) {
        const peers = this._pubsub?.getSubscribers(topic);
        if (!peers)
            return;
        const peerId = peers[Math.floor(Math.random() * peers.length)].toString();
        const connection = await this._node?.dial([multiaddr(peerId)]);
        const stream = (await connection?.newStream(protocols));
        stringToStream(stream, message);
    }
    addPubsubEventListener(type, event) {
        this._pubsub?.addEventListener(type, event);
    }
    addMessageHandler(protocol, handler) {
        this._node?.handle(protocol, handler);
    }
}
